package com.mcwl.web.controller.resource;

import com.mcwl.common.annotation.Anonymous;
import com.mcwl.common.core.controller.BaseController;
import com.mcwl.common.core.domain.AjaxResult;
import com.mcwl.common.core.page.PageDomain;
import com.mcwl.common.core.page.TableDataInfo;
import com.mcwl.common.domain.IdsParam;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.resource.domain.ModelProduct;
import com.mcwl.resource.domain.dto.ModelImagePageRes;
import com.mcwl.resource.domain.request.RequestModel;
import com.mcwl.resource.service.*;
import com.mcwl.web.controller.common.OssUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * 模型
 *
 * @Author：ChenYan
 * @Project：McWl
 * @Package：com.mcwl.web.controller.resource
 * @Filename：MallProductController
 * @Description 商品
 * @Date：2024/12/31 10:48
 */
@Api(tags = "模型")
@RestController
@RequestMapping("/model")
public class MallProductController extends BaseController {


    @Autowired
    private ModelService modelService;

    @Autowired
    private ModelLikeService modelLikeService;

    @Autowired
    private ModelImageService modelImageService;

    @Autowired
    private WorkFlowService workFlowService;

    @Autowired
    private WorkFlowLikeService workFlowLikeService;

    @Autowired
    private ModelImageLikeService modelImageLikeService;


    /***
     *
     * 图片
     * @param file
     * @return
     */
    @ApiModelProperty("图片")
    @Anonymous
    @PostMapping("/file")
    public AjaxResult Malifile(@RequestParam MultipartFile file) {

        String s = OssUtil.uploadMultipartFile(file);
        String fileName = file.getOriginalFilename();

        Map<String, String> map = new HashMap<>();
        map.put("fileName", fileName);
        map.put("url", s);

        return AjaxResult.success(map);
    }


    /***
     *
     * zip
     * @param file
     * @return
     */
    @ApiOperation(value = "zip")
    @PostMapping("/zipUrlFile")
    public AjaxResult zipUrlFile(@RequestParam MultipartFile file) {
        String s = OssUtil.uploadMultipartFile(file);
        return AjaxResult.success(s);
    }


    /***
     *
     * 下载zip
     * @param file
     * @return
     */
    @ApiOperation(value = "zip")
    @PostMapping("/zipUrl")
    public AjaxResult zipUrl(@RequestParam MultipartFile file) {
        String s = OssUtil.uploadMultipartFile(file);
        return AjaxResult.success(s);
    }


    /**
     * 模型列表
     */
    @ApiOperation(value = "模型列表")
    @PostMapping("/list")
    public TableDataInfo list(@RequestBody ModelImagePageRes imagePageRes) {

        return modelService.listByPage(imagePageRes);
    }


    @ApiOperation(value = "模型详情")
    @PostMapping("finbyid")
    public AjaxResult finbyid(@RequestBody ModelProduct modelVersion) {
        ModelProduct modelVersion1 = modelService.getById(modelVersion.getId());
        return AjaxResult.success(modelVersion1);
    }


    @ApiOperation(value = "添加模型")
    @PostMapping("/insert")
    public AjaxResult addupdateModel(@RequestBody RequestModel requestModel) {
        ModelProduct modelProduct = requestModel.getModelProduct();
        Long userId = SecurityUtils.getUserId();
        modelProduct.setUserId(userId);
        modelProduct.setCreateBy(getUsername());
        return modelService.addModel(requestModel);
    }

    @ApiOperation(value = "修改模型")
    @PostMapping("/update")
    public AjaxResult updateModel(@RequestBody RequestModel requestModel) {
        ModelProduct modelProduct = requestModel.getModelProduct();

        Long userId = SecurityUtils.getUserId();
        modelProduct.setUserId(userId);
        modelProduct.setUpdateBy(getUsername());
        modelService.updaModel(requestModel);

        return AjaxResult.success("修改成功");
    }


    @ApiOperation(value = "删除模型")
    @GetMapping("delete")
    public AjaxResult delete(@Valid @NotNull(message = "模型id不能为空") Long id) {
        modelService.removeById(id);
        return AjaxResult.success();
    }

    /**
     * 查询模型详情
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "查询模型详情")
    @GetMapping("/selectModelById")
    public AjaxResult selectModelById(@RequestParam Long id) {

        return modelService.selectModelById(id);
    }


    /**
     * 设置模型置顶状态
     *
     * @param id
     * @param isTop
     * @return
     */
    @ApiOperation(value = "设置模型置顶状态")
    @GetMapping("/{id}/top")
    public AjaxResult setModelTop(@PathVariable Long id, @RequestParam boolean isTop) {
        modelService.setModelTop(id, isTop);
        return AjaxResult.success();
    }


    /**
     * 获取置顶的模型列表
     *
     * @return
     */
    @ApiOperation(value = "获取置顶的模型列表")
    @GetMapping
    public ResponseEntity<List<ModelProduct>> fetchModelsSortedByTopStatus() {
        List<ModelProduct> models = modelService.fetchModelsSortedByTopStatus();
        return ResponseEntity.ok(models);
    }


    /**
     * 我的发布-模型列表
     */
    @ApiOperation(value = "我的发布-模型列表")
    @PostMapping("/selectByUserIdModel")
    public TableDataInfo selectByUserIdModel(@RequestBody ModelImagePageRes imagePageRes) {

        return modelService.listByPage(imagePageRes);
    }

    /**
     * 我的发布-工作流列表
     */
    @ApiOperation(value = "我的发布-工作流列表")
    @PostMapping("/selectByUserIdWorkFlow")
    public TableDataInfo selectByUserIdWorkFlow(@RequestBody ModelImagePageRes imagePageRes) {

        return workFlowService.listByPage(imagePageRes);
    }

    /**
     * 我的发布-图片列表
     */
    @ApiOperation(value = "我的发布-图片列表")
    @PostMapping("/selectByUserIdImage")
    public TableDataInfo selectByUserIdImage(@RequestBody ModelImagePageRes imagePageRes) {

        return modelImageService.listByPage(imagePageRes);
    }

    /**
     * 我的点赞-模型列表
     */
    @ApiOperation(value = "我的点赞-模型列表")
    @PostMapping("/likeModel")
    public TableDataInfo likeModel(@RequestBody PageDomain pageDomain) {
        return modelLikeService.listByPage(pageDomain);
    }

    /**
     * 我的点赞-工作流列表
     */
    @ApiOperation(value = "我的点赞-工作流列表")
    @PostMapping("/likeWorkFlow")
    public TableDataInfo likeWorkFlow(@RequestBody PageDomain pageDomain) {
        return workFlowLikeService.listByPage(pageDomain);
    }

    /**
     * 我的点赞-图片列表
     */
    @ApiOperation(value = "我的点赞-图片列表")
    @PostMapping("/likeImage")
    public TableDataInfo likeImage(@RequestBody PageDomain pageDomain) {
        return modelImageLikeService.listByPage(pageDomain);
    }


}
