package com.mcwl.web.controller.resource;

import com.mcwl.common.core.domain.AjaxResult;
import com.mcwl.resource.domain.WorkFlowComment;
import com.mcwl.resource.domain.vo.WorkFlowCommentVo;
import com.mcwl.resource.service.WorkFlowCommentLikeService;
import com.mcwl.resource.service.WorkFlowCommentService;
import com.mcwl.resource.service.WorkFlowLikeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * 工作流评论
 * @Author：ChenYan
 * @Project：McWl
 * @Package：com.mcwl.web.controller.resource
 * @Filename：ModelCommentcontroller
 * @Description TODO
 * @Date：2025/1/12 11:36
 */
@Api(tags = "工作流评论")
@RequestMapping("/WorkFlowComment")
@RestController
public class WorkFlowCommentController {

    @Autowired
    private WorkFlowLikeService workFlowLikeService;
    @Autowired
    private WorkFlowCommentService workFlowCommentService;
    @Autowired
    private WorkFlowCommentLikeService workFlowCommentLikeService;



    /**
     * 工作流点赞/取消
     */
    @ApiOperation(value = "工作流点赞/取消")
    @GetMapping("/like/{workFlowId}")
    public AjaxResult like(@PathVariable Long workFlowId) {
        workFlowLikeService.like(workFlowId);
        return AjaxResult.success();
    }


    /**
     * 工作流评论发布
     */
    @ApiOperation(value = "工作流评论发布")
    @PostMapping("/comment")
    public AjaxResult comment(@RequestBody WorkFlowComment modelComment) {
        workFlowCommentService.comment(modelComment);
        return AjaxResult.success();
    }

    /**
     * 工作流评论点赞/取消
     */
    @ApiOperation(value = "工作流评论点赞/取消")
    @GetMapping("/commentLike/{commentId}")
    public AjaxResult commentLike(@PathVariable Long commentId) {
        workFlowCommentLikeService.like(commentId);
        return AjaxResult.error();
    }



    /**
     * 获取工作流评论
     */
    @ApiOperation(value = "获取工作流评论")
    @GetMapping("/comment/{modelId}")
    public AjaxResult getComment(@PathVariable @NotNull(message = "模型id不能为空") Long modelId) {
        List<WorkFlowCommentVo> modelCommentList = workFlowCommentService.getComment(modelId);
        return AjaxResult.success(modelCommentList);
    }


    /**
     * 删除工作流评论
     */
    @ApiOperation(value = "删除工作流评论")
    @GetMapping("/commentDelete/{commentId}")
    public AjaxResult commentDelete(@PathVariable @NotNull(message = "评论id不能为空") Long commentId) {
        workFlowCommentService.removeById(commentId);
        return AjaxResult.success();
    }


}
