package com.mcwl.pay.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.UUID;
import cn.hutool.extra.mail.MailUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONObject;
import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.AlipayConfig;
import com.alipay.api.DefaultAlipayClient;
import com.alipay.api.diagnosis.DiagnosisUtils;
import com.alipay.api.domain.AlipayFundAccountQueryModel;
import com.alipay.api.domain.AlipayFundTransUniTransferModel;
import com.alipay.api.domain.Participant;
import com.alipay.api.request.AlipayFundAccountQueryRequest;
import com.alipay.api.request.AlipayFundTransUniTransferRequest;
import com.alipay.api.request.AlipaySystemOauthTokenRequest;
import com.alipay.api.response.AlipayFundAccountQueryResponse;
import com.alipay.api.response.AlipayFundTransUniTransferResponse;
import com.alipay.easysdk.factory.Factory;
import com.alipay.easysdk.kernel.Config;
import com.alipay.easysdk.payment.facetoface.models.AlipayTradePrecreateResponse;
import com.mcwl.common.core.domain.AjaxResult;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.core.redis.RedisCache;
import com.mcwl.common.exception.ServiceException;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.common.utils.ShareCodeUtils;
import com.mcwl.memberCenter.domain.MemberLevel;
import com.mcwl.memberCenter.service.MemberLevelService;
import com.mcwl.pay.config.AliConfig;
import com.mcwl.pay.domain.OrderTrade;
import com.mcwl.pay.domain.OrderTradeDto;
import com.mcwl.pay.domain.vo.PayVo;
import com.mcwl.pay.service.AliPayService;
import com.mcwl.system.domain.SysUserPayAccount;
import com.mcwl.system.domain.SysUserPayAccountLog;
import com.mcwl.system.service.ISysUserPayAccountLogService;
import com.mcwl.system.service.ISysUserPayAccountService;
import com.mcwl.system.service.ISysUserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Objects;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * 支付宝支付
 */
@Component
@Slf4j
public class AliPayServiceImpl implements AliPayService {

    @Autowired
    private RedisCache redisCache;


    @Autowired
    private MemberLevelService memberLevelService;


    @Autowired
    private AliConfig aliConfig;

    @Autowired
    private ISysUserPayAccountService sysUserPayAccountService;

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private ISysUserPayAccountLogService sysUserPayAccountLogService;


    public AliPayServiceImpl(Config config) {
        Factory.setOptions(config);
    }


    /**
     * 调用支付宝预下订单接口
     *
     * @param orderTradeDto 订单实体
     * @return 二维码url
     * @throws Exception
     */
//    public String orderPay(OrderTradeDto orderTradeDto) throws Exception {
//        Integer productId = orderTradeDto.getProductId();
//        if (!Optional.ofNullable(productId).isPresent()) {
//            throw new ServiceException("mallProductId不能为空");
//        }
//
//        MallProduct mallProduct = mallProductService.getById(productId);
//        if (!Optional.ofNullable(mallProduct).isPresent()) {
//            throw new ServiceException("mallProduct不存在");
//        }
//
//        // 设置orderTrade信息
//        OrderTrade tradeEntity = new OrderTrade();
//        BeanUtil.copyProperties(orderTradeDto, tradeEntity);
//        tradeEntity.setCode(UUID.randomUUID().toString(true).substring(0, 30));
//        tradeEntity.setUserId(SecurityUtils.getUserId());
//        tradeEntity.setUserName(SecurityUtils.getUsername());
//        tradeEntity.setProductName(mallProduct.getProductName());
//
//        //调用支付宝的接口
//        AlipayTradePrecreateResponse payResponse = Factory.Payment.FaceToFace()
//                .preCreate(mallProduct.getProductName(),
//                        tradeEntity.getCode() + "_product",
//                        orderTradeDto.getPaymentAmount().toString());
//        // 缓存到redis
//        redisCache.setCacheObject(tradeEntity.getCode() + "_product", JSONUtil.toJsonStr(tradeEntity), 3, TimeUnit.MINUTES);
////        AlipayTradePrecreateResponse payResponse = Factory.Payment.FaceToFace().preCreate("订单主题：Mac笔记本", "LS123qwe123", "19999");
//        //参照官方文档响应示例，解析返回结果
//        String httpBodyStr = payResponse.getHttpBody();
//        JSONObject jsonObject = JSONObject.parseObject(httpBodyStr);
//        return jsonObject.getJSONObject("alipay_trade_precreate_response").get("qr_code").toString();
//    }


    /**
     * 调用支付宝预下订单接口
     *
     * @param orderTradeDto 订单实体
     * @return 二维码url
     * @throws Exception
     */
    @Override
    public PayVo memberPay(OrderTradeDto orderTradeDto) throws Exception {

        // 会员等级id
        Integer productId = orderTradeDto.getProductId();

        if (!Optional.ofNullable(productId).isPresent()) {
            throw new ServiceException("memberLevelId不能为空");
        }

        MemberLevel memberLevel = memberLevelService.getById(productId);

        if (!Optional.ofNullable(memberLevel).isPresent()) {
            throw new ServiceException("memberLevel不存在");
        }

        // 设置orderTrade信息
        OrderTrade tradeEntity = new OrderTrade();
        BeanUtil.copyProperties(orderTradeDto, tradeEntity);
        tradeEntity.setCode(UUID.randomUUID().toString(true).substring(0, 30));
        tradeEntity.setUserId(SecurityUtils.getUserId());
        tradeEntity.setUserName(SecurityUtils.getUsername());
        tradeEntity.setProductName(memberLevel.getMemberName());

        //调用支付宝的接口
        AlipayTradePrecreateResponse payResponse = Factory.Payment.FaceToFace()
                // 设置过期时
                .preCreate(memberLevel.getMemberName(),
                        tradeEntity.getCode() + "_member",
                        orderTradeDto.getPaymentAmount().toString());
        // 缓存到redis
        redisCache.setCacheObject(tradeEntity.getCode() + "_member", JSONUtil.toJsonStr(tradeEntity), 3, TimeUnit.MINUTES);
        redisCache.setCacheObject(tradeEntity.getCode() + "_member" + "_promotionId", orderTradeDto.getPromotionId(), 3, TimeUnit.MINUTES);
//        AlipayTradePrecreateResponse payResponse = Factory.Payment.FaceToFace().preCreate("订单主题：Mac笔记本", "LS123qwe123", "19999");
        //参照官方文档响应示例，解析返回结果
        String httpBodyStr = payResponse.getHttpBody();
        JSONObject jsonObject = JSONObject.parseObject(httpBodyStr);

        PayVo payVo = new PayVo();
        payVo.setOrderNo(tradeEntity.getCode());
        payVo.setUrl(jsonObject.getJSONObject("alipay_trade_precreate_response").get("qr_code").toString());
        return payVo;
    }


    /**
     * 调用支付宝预下订单接口
     *
     * @param paymentAmount 充值金额
     * @return 二维码url
     * @throws Exception
     */
    @Override
    public PayVo pointsPay(Double paymentAmount) throws Exception {
        // 设置orderTrade信息
        OrderTrade tradeEntity = new OrderTrade();
        tradeEntity.setCode(UUID.randomUUID().toString(true).substring(0, 30));
        tradeEntity.setUserId(SecurityUtils.getUserId());
        tradeEntity.setProductId(-1);
        tradeEntity.setProductName("积分充值");
        tradeEntity.setUserName(SecurityUtils.getUsername());
        tradeEntity.setPaymentAmount(paymentAmount.intValue());

        //调用支付宝的接口
        AlipayTradePrecreateResponse payResponse = Factory.Payment.FaceToFace()
                .preCreate(tradeEntity.getProductName(),
                        tradeEntity.getCode() + "_points",
                        tradeEntity.getPaymentAmount().toString());
        // 缓存到redis
        redisCache.setCacheObject(tradeEntity.getCode() + "_points", JSONUtil.toJsonStr(tradeEntity), 3, TimeUnit.MINUTES);
        //参照官方文档响应示例，解析返回结果
        String httpBodyStr = payResponse.getHttpBody();
        JSONObject jsonObject = JSONObject.parseObject(httpBodyStr);

        String url = jsonObject.getJSONObject("alipay_trade_precreate_response").get("qr_code").toString();
        PayVo payVo = new PayVo();
        payVo.setOrderNo(tradeEntity.getCode());
        payVo.setUrl(url);

        return payVo;
    }

    /**
     * 支付宝转账方法
     *
     * @param outBizNo 外部业务单号
     * @param amount   转账金额
     * @return 返回支付宝转账响应的内容
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public AjaxResult fetch(String outBizNo, String amount) throws AlipayApiException {
        Long userId = SecurityUtils.getUserId();
        SysUser sysUser = sysUserService.selectUserById(userId);
        if (sysUser.getWallet() < Double.parseDouble(amount)) {
            return AjaxResult.error("钱包余额不足");
        }

        // 初始化SDK
        AlipayClient alipayClient = new DefaultAlipayClient(getAlipayConfig());

        // 构造请求参数以调用接口
        AlipayFundTransUniTransferRequest request = new AlipayFundTransUniTransferRequest();
        AlipayFundTransUniTransferModel model = new AlipayFundTransUniTransferModel();

        // 设置商家侧唯一订单号
        model.setOutBizNo(outBizNo);

        // 设置订单总金额
        model.setTransAmount(amount);

        // 设置描述特定的业务场景
        model.setBizScene("DIRECT_TRANSFER");

        // 设置业务产品码
        model.setProductCode("TRANS_ACCOUNT_NO_PWD");

        // 设置转账业务的标题
        model.setOrderTitle("提现");

        // 查询用户支付宝账户
        SysUserPayAccount sysUserPayAccount = sysUserPayAccountService
                .lambdaQuery()
                .eq(SysUserPayAccount::getUserId, userId)
                .eq(SysUserPayAccount::getType, 0)
                .one();
        if (Objects.isNull(sysUserPayAccount)) {
            return AjaxResult.error("请先绑定支付宝账号");
        }

        // 收款方信息
        Participant payeeInfo = new Participant();

        // 沙箱环境
        payeeInfo.setIdentity("2088722057445020");
        payeeInfo.setIdentityType("ALIPAY_USER_ID");

//        线上环境
//        payeeInfo.setIdentity(sysUserPayAccount.getOpenId());
//        payeeInfo.setIdentityType("ALIPAY_OPEN_ID");
        model.setPayeeInfo(payeeInfo);

        request.setBizModel(model);

        // 证书用：certificateExecute(request)，密钥用：execute(request)
        AlipayFundTransUniTransferResponse response = alipayClient.execute(request);
        System.out.println(response.getBody());

        if (response.isSuccess()) {
            // 记录日志和修改用户钱包
            this.addLogAndUpdateUserWallet(sysUser, amount);
            return AjaxResult.success("转账成功");
        }

        if (response.getSubCode().equals("PAYER_BALANCE_NOT_ENOUGH")) {
            // 账户余额不足，发送邮件通知
            ArrayList<String> tos = CollUtil.newArrayList("2119157836@qq.com");

            String content = String.format("账户余额不足：用户%s提现%s", sysUser.getUserName(), amount);
            MailUtil.send(tos, "上海辰风互娱", content, false);
            throw new ServiceException("系统繁忙，请稍后重试");
        }

        throw new ServiceException("转账失败：" + response.getSubMsg());
    }

    @Override
    public String balance() throws AlipayApiException {
        // 初始化SDK
        AlipayClient alipayClient = new DefaultAlipayClient(getAlipayConfig());

        // 构造请求参数以调用接口
        AlipayFundAccountQueryRequest request = new AlipayFundAccountQueryRequest();
        AlipayFundAccountQueryModel model = new AlipayFundAccountQueryModel();

        // 应用userId
        model.setAlipayUserId("2088721057420516");

        // 设置查询的账号类型
        model.setAccountType("ACCTRANS_ACCOUNT");

        request.setBizModel(model);
        AlipayFundAccountQueryResponse response = alipayClient.execute(request);

        if (response.isSuccess()) {
            System.out.println("调用成功");
            return response.getAvailableAmount();
        } else {
            System.out.println("调用失败");
            return "fail";
        }

    }

    @Override
    public String bindingCallback(String authCode, String state) {
        try {
            Long userId = ShareCodeUtils.codeToId(state);
            if (Objects.isNull(userId)) {
                return "fail";
            }
            AlipayClient alipayClient = new DefaultAlipayClient(getAlipayConfig());
            AlipaySystemOauthTokenRequest request = new AlipaySystemOauthTokenRequest();
            request.setCode(authCode);
            request.setGrantType("authorization_code");
            com.alipay.api.response.AlipaySystemOauthTokenResponse response = alipayClient.execute(request);
            if (response.isSuccess()) {
                // 支付宝用户唯一ID
                String openId = response.getOpenId();
                // 判断是否已经绑定过
                SysUserPayAccount sysUserPayAccount = sysUserPayAccountService.lambdaQuery()
                        .eq(SysUserPayAccount::getUserId, userId)
                        .eq(SysUserPayAccount::getType, 0)
                        .one();
                if (Objects.nonNull(sysUserPayAccount)) {
                    // 已经绑定过，直接返回
                    return "success";
                }
                System.out.println("绑定成功！openId：" + openId);
                // 将openId与当前商城用户绑定（保存到数据库）
                SysUserPayAccount userPayAccount = new SysUserPayAccount();
                userPayAccount.setUserId(userId);
                userPayAccount.setOpenId(openId);
                userPayAccount.setType(0);
                sysUserPayAccountService.save(userPayAccount);
                return "success";
            } else {
                System.out.println("绑定失败：" + response.getSubMsg());
                return "fail";
            }
        } catch (AlipayApiException e) {
            return "fail";
        }

    }


    private void addLogAndUpdateUserWallet(SysUser sysUser, String amount) {
        BigDecimal wallet = new BigDecimal(sysUser.getWallet().toString());
        BigDecimal amountBigDecimal = new BigDecimal(amount);
        sysUser.setWallet(wallet.subtract(amountBigDecimal).doubleValue());
        sysUserService.updateUser(sysUser);

        // 添加记录
        SysUserPayAccountLog sysUserPayAccountLog = new SysUserPayAccountLog();
        sysUserPayAccountLog.setUserId(sysUser.getUserId());
        sysUserPayAccountLog.setAmount(Double.valueOf(amount));
        sysUserPayAccountLog.setAccount("支付宝");
        sysUserPayAccountLogService.save(sysUserPayAccountLog);
    }


    private AlipayConfig getAlipayConfig() {
        String privateKey = aliConfig.getPrivateKey();
        String alipayPublicKey = aliConfig.getPublicKey();
        AlipayConfig alipayConfig = new AlipayConfig();
        alipayConfig.setServerUrl(aliConfig.getGatewayUrl());
        alipayConfig.setAppId(aliConfig.getAppId());
        alipayConfig.setPrivateKey(privateKey);
        alipayConfig.setFormat("json");
        alipayConfig.setAlipayPublicKey(alipayPublicKey);
        alipayConfig.setCharset("UTF-8");
        alipayConfig.setSignType("RSA2");
        return alipayConfig;
    }
}
