package com.mcwl.resource.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mcwl.common.exception.ServiceException;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.resource.domain.WorkFlowComment;
import com.mcwl.resource.domain.WorkFlowCommentLike;
import com.mcwl.resource.mapper.WorkFlowCommentLikeMapper;
import com.mcwl.resource.mapper.WorkFlowCommentMapper;
import com.mcwl.resource.service.WorkFlowCommentLikeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.Objects;

/**
 * @Author：ChenYan
 * @Project：McWl
 * @Package：com.mcwl.resource.service.impl
 * @Filename：ModelCommentLikeServiceImpl
 * @Description TODO
 * @Date：2025/1/12 12:01
 */
@Service
public class WorkFlowCommentLikeServiceImpl extends ServiceImpl<WorkFlowCommentLikeMapper, WorkFlowCommentLike> implements WorkFlowCommentLikeService {

    @Autowired
    private WorkFlowCommentMapper workFlowCommentMapper;

    @Override
    @Transactional
    public void like(Long commentId) {
        WorkFlowComment workFlowComment = workFlowCommentMapper.selectById(commentId);
        if (Objects.isNull(workFlowComment)) {
            throw new ServiceException("该评论不存在");
        }
        Long userId = SecurityUtils.getUserId();
        WorkFlowCommentLike workFlowCommentLike = baseMapper.getLikeComment(userId, commentId);
        if (Objects.nonNull(workFlowCommentLike)) {
            if (Objects.equals(workFlowComment.getDelFlag(), "0")) {
                workFlowCommentLike.setDelFlag("1");
                workFlowComment.setLikeNum(workFlowComment.getLikeNum() - 1);
            } else {
                workFlowCommentLike.setDelFlag("0");
                workFlowComment.setLikeNum(workFlowComment.getLikeNum() + 1);
            }
            // 更新点赞记录
            baseMapper.updateById(workFlowCommentLike);
            // 更新图片评论点赞数
            workFlowCommentMapper.updateById(workFlowComment);
            return;
        }

        // 添加点赞记录
        workFlowCommentLike = new WorkFlowCommentLike();
        workFlowCommentLike.setUserId(userId);
        workFlowCommentLike.setWorkFlowCommentId(commentId);
        workFlowCommentLike.setCreateBy(SecurityUtils.getUsername());
        workFlowCommentLike.setUpdateBy(SecurityUtils.getUsername());
        workFlowCommentLike.setUpdateTime(new Date());
        baseMapper.insert(workFlowCommentLike);

        // 更新模型点赞数
        workFlowComment.setLikeNum(workFlowComment.getLikeNum() + 1);
        workFlowCommentMapper.updateById(workFlowComment);
    }

}
